%define gtm_root /opt/lsb-gtm

%ifarch x86_64
%define gtm_arch x86_64
%else
%define gtm_arch i686
%endif

Name:		openvista-gtm-integration
Version:	0.8.6
Release:	1
Summary:	Utilities for configuring and managing OpenVista
License:	AGPL
Vendor:		Medsphere Systems Corporation
URL:		https://medsphere.org/community/project/gtm
Group:		Applications/System
Packager:	Jonathan Tai <jon.tai@medsphere.com>

Source0:	openvista-gtm-integration-%{version}.tar.bz2
Source1:	webdav.conf
Source2:	openvista
BuildRoot:	%{_tmppath}/%{name}-root

%description
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

%prep
%setup

%build
cd src/libopenvista

for gtm_version in V5.3-003 V5.3-004 V5.3-004A; do
    export gtm_dist=%{gtm_root}/${gtm_version}_%{gtm_arch}
    ./compile.sh
    mv libopenvista.so libopenvista.so-${gtm_version}
    mv openvista.xc openvista.xc-${gtm_version}
done

cd ../..

cd src/gtmsignal
./compile.sh

cd ../..

cd src/ovauth
./compile.sh

%install
rm -rf %{buildroot}

for gtm_version in V5.3-003 V5.3-004 V5.3-004A; do
    export gtm_dist=%{gtm_root}/${gtm_version}_%{gtm_arch}
    install -d %{buildroot}$gtm_dist
    install -m 550 src/libopenvista/libopenvista.so-${gtm_version} %{buildroot}$gtm_dist/libopenvista.so
    install -m 440 src/libopenvista/openvista.xc-${gtm_version} %{buildroot}$gtm_dist/openvista.xc
    install -m 4550 src/gtmsignal/gtmsignal %{buildroot}$gtm_dist/
done

install -d %{buildroot}/usr/lib/openvista
install scripts/usr/lib/openvista/functions %{buildroot}/usr/lib/openvista

install -d %{buildroot}%{_datadir}/openvista
for config in scripts/usr/share/openvista/*; do
    install -m 644 $config %{buildroot}%{_datadir}/openvista/
done

install -d %{buildroot}%{_sbindir}/
for script in scripts/usr/sbin/*; do
    install $script %{buildroot}%{_sbindir}/
done

install -d %{buildroot}%{_bindir}/
for script in scripts/usr/bin/*; do
    install $script %{buildroot}%{_bindir}/
done

install -d %{buildroot}%{_mandir}/man8/
for man in docs/man8/*; do
    install -m 644 $man %{buildroot}%{_mandir}/man8/
    gzip %{buildroot}%{_mandir}/man8/$(basename $man)
done

install -d %{buildroot}%{_mandir}/man1/
for man in docs/man1/*; do
    install -m 644 $man %{buildroot}%{_mandir}/man1/
    gzip %{buildroot}%{_mandir}/man1/$(basename $man)
done

install -d %{buildroot}%{_sysconfdir}/rc.d/init.d/
install scripts/etc/init.d/openvista-databases %{buildroot}%{_sysconfdir}/rc.d/init.d/
install scripts/etc/init.d/openvista %{buildroot}%{_sysconfdir}/rc.d/init.d/

install -d %{buildroot}%{_sysconfdir}/bash_completion.d/
install -m 644 scripts/etc/bash_completion.d/openvista %{buildroot}%{_sysconfdir}/bash_completion.d/

install -d %{buildroot}%{_sysconfdir}/cron.d/
install -m 644 scripts/etc/cron.d/openvista %{buildroot}%{_sysconfdir}/cron.d/

install -d %{buildroot}%{_sysconfdir}/cron.daily/
install scripts/etc/cron.daily/openvista %{buildroot}%{_sysconfdir}/cron.daily/

install -d %{buildroot}/opt/openvista

install -d %{buildroot}%{_sysconfdir}/httpd/conf.d
install -m 644 %{SOURCE1} %{buildroot}%{_sysconfdir}/httpd/conf.d/

install -d %{buildroot}%{_sysconfdir}/httpd/conf
touch %{buildroot}%{_sysconfdir}/httpd/conf/webdav.htpasswd

install -d %{buildroot}/var/www/html/webdav

install -d %{buildroot}%{_sysconfdir}/pam.d
install -m 644 %{SOURCE2} %{buildroot}%{_sysconfdir}/pam.d/

install -d %{buildroot}/sbin
install -m 4550 src/ovauth/ovauth %{buildroot}/sbin

install -d %{buildroot}%{_sysconfdir}/munin/plugin-conf.d/
install scripts/etc/munin/plugin-conf.d/openvista %{buildroot}%{_sysconfdir}/munin/plugin-conf.d/

install -d %{buildroot}%{_datadir}/munin/plugins/
for plugin in scripts/usr/share/munin/plugins/*; do
    install $plugin %{buildroot}%{_datadir}/munin/plugins/
done

%clean
rm -rf %{buildroot}





%define gtm_version V5.3-003
%define gtm_dist %{gtm_root}/%{gtm_version}_%{gtm_arch}

%package -n openvista-libs-5.3003
Release:	1
Summary:	GT.M External Calls for OpenVista
Group:		Applications/System

Provides:	openvista-libs

Requires(post):		/usr/sbin/semanage, /sbin/restorecon
Requires(postun):	/usr/sbin/semanage

Requires:	fis-gtm-5.3003
BuildRequires:	fis-gtm-5.3003

Obsoletes:	openvista-libs

%description -n openvista-libs-5.3003
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

This package provides a shared library that OpenVista code running in GT.M
calls out to for PAM authentication and duplicating a device onto a spawned
child's stdin/stdout.  This package also provides an suid wrapper around GT.M
utilities used to send signals to other GT.M processes.

The shared libraries in this package were compiled against GT.M %{gtm_version}.

%files -n openvista-libs-5.3003
%defattr(-,root,root)
%doc README
%doc COPYING
%doc COPYING.AGPL
%doc NEWS
%defattr(-,bin,gtm)
%{gtm_dist}/libopenvista.so
%{gtm_dist}/openvista.xc
%defattr(-,root,gtm)
%{gtm_dist}/gtmsignal

%post -n openvista-libs-5.3003
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
/usr/sbin/semanage fcontext -a -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
/sbin/restorecon %{gtm_dist}/libopenvista.so || :

%postun -n openvista-libs-5.3003
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
if [ $1 -eq 0 ]; then
    /usr/sbin/semanage fcontext -d -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
fi





%define gtm_version V5.3-004
%define gtm_dist %{gtm_root}/%{gtm_version}_%{gtm_arch}

%package -n openvista-libs-5.3004
Release:	1
Summary:	GT.M External Calls for OpenVista
Group:		Applications/System

Provides:	openvista-libs

Requires(post):		/usr/sbin/semanage, /sbin/restorecon
Requires(postun):	/usr/sbin/semanage

Requires:	fis-gtm-5.3004
BuildRequires:	fis-gtm-5.3004

Obsoletes:	openvista-libs

%description -n openvista-libs-5.3004
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

This package provides a shared library that OpenVista code running in GT.M
calls out to for PAM authentication and duplicating a device onto a spawned
child's stdin/stdout.  This package also provides an suid wrapper around GT.M
utilities used to send signals to other GT.M processes.

The shared libraries in this package were compiled against GT.M %{gtm_version}.

%files -n openvista-libs-5.3004
%defattr(-,root,root)
%doc README
%doc COPYING
%doc COPYING.AGPL
%doc NEWS
%defattr(-,bin,gtm)
%{gtm_dist}/libopenvista.so
%{gtm_dist}/openvista.xc
%defattr(-,root,gtm)
%{gtm_dist}/gtmsignal

%post -n openvista-libs-5.3004
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
/usr/sbin/semanage fcontext -a -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
/sbin/restorecon %{gtm_dist}/libopenvista.so || :

%postun -n openvista-libs-5.3004
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
if [ $1 -eq 0 ]; then
    /usr/sbin/semanage fcontext -d -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
fi





%define gtm_version V5.3-004A
%define gtm_dist %{gtm_root}/%{gtm_version}_%{gtm_arch}

%package -n openvista-libs-5.3004a
Release:	1
Summary:	GT.M External Calls for OpenVista
Group:		Applications/System

Provides:	openvista-libs

Requires(post):		/usr/sbin/semanage, /sbin/restorecon
Requires(postun):	/usr/sbin/semanage

Requires:	fis-gtm-5.3004a
BuildRequires:	fis-gtm-5.3004a

Obsoletes:	openvista-libs

%description -n openvista-libs-5.3004a
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

This package provides a shared library that OpenVista code running in GT.M
calls out to for PAM authentication and duplicating a device onto a spawned
child's stdin/stdout.  This package also provides an suid wrapper around GT.M
utilities used to send signals to other GT.M processes.

The shared libraries in this package were compiled against GT.M %{gtm_version}.

%files -n openvista-libs-5.3004a
%defattr(-,root,root)
%doc README
%doc COPYING
%doc COPYING.AGPL
%doc NEWS
%defattr(-,bin,gtm)
%{gtm_dist}/libopenvista.so
%{gtm_dist}/openvista.xc
%defattr(-,root,gtm)
%{gtm_dist}/gtmsignal

%post -n openvista-libs-5.3004a
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
/usr/sbin/semanage fcontext -a -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
/sbin/restorecon %{gtm_dist}/libopenvista.so || :

%postun -n openvista-libs-5.3004a
# See http://fedoraproject.org/wiki/PackagingDrafts/SELinux
if [ $1 -eq 0 ]; then
    /usr/sbin/semanage fcontext -d -t textrel_shlib_t %{gtm_dist}/libopenvista.so 2>/dev/null || :
fi





%package -n openvista-utils
Release:	2
Summary:	Utilities for configuring and managing OpenVista
Group:		Applications/System

Requires(pre):		/usr/sbin/useradd
Requires(post):		/sbin/chkconfig, /sbin/service
Requires(preun):	/sbin/chkconfig
Requires(postun):	/usr/sbin/userdel, /sbin/service

Requires:	/bin/bash, coreutils, openssh, /usr/bin/logger, /usr/bin/rsync, /bin/tar, /usr/bin/bzip2, /usr/bin/find, vixie-cron
Requires:	openvista-libs
Requires:	httpd, mod_ssl, pam
BuildRequires:	pam-devel

# RPM should have a suggests tag...
# Suggests:	/usr/bin/puttygen, bash-completion, /usr/bin/pbzip2

Obsoletes:	openvista-webdav-config

%description -n openvista-utils
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

This package provides command-line utilities that aid in configuring and
managing OpenVista instances. 

%files -n openvista-utils
%defattr(-,root,root)
%doc README
%doc COPYING
%doc COPYING.AGPL
%doc NEWS
/usr/lib/openvista/functions
%{_datadir}/openvista/*
%{_sbindir}/*
%{_bindir}/*
%{_mandir}/man8/*
%{_mandir}/man1/*
%{_sysconfdir}/rc.d/init.d/openvista-databases
%{_sysconfdir}/rc.d/init.d/openvista
%{_sysconfdir}/bash_completion.d/openvista
%config %{_sysconfdir}/cron.d/openvista
%{_sysconfdir}/cron.daily/openvista
%attr(2750,root,openvista) %dir /opt/openvista
%{_sysconfdir}/httpd/conf.d/webdav.conf
%config %{_sysconfdir}/httpd/conf/webdav.htpasswd
%dir /var/www/html/webdav
%{_sysconfdir}/pam.d/openvista
%attr(4550,root,openvista) /sbin/ovauth

%pre -n openvista-utils
/usr/sbin/groupadd openvista 2> /dev/null || :
/usr/sbin/useradd -g openvista -G gtm openvista 2> /dev/null || :

# update lock space in existing databases if upgrading
%triggerun -n openvista-utils -- openvista-utils <= 0.8.3-2
echo "Increasing lock space of existing databases..."
for instance in /opt/openvista/*; do
    # update global directory
    if [ -L ${instance}/gtm ]; then
        gtm_dist=${instance}/gtm gtmgbldir=${instance}/globals/mumps.gld gtmroutines=${instance}/gtm ${instance}/gtm/mumps -run ^GDE > /dev/null 2>&1 <<EOF
CHANGE -SEGMENT DEFAULT -LOCK_SPACE=400
CHANGE -SEGMENT SCRATCH -LOCK_SPACE=400
TEMPLATE -SEGMENT -LOCK_SPACE=400
EXIT
EOF

        # update actual database files
        gtm_dist=${instance}/gtm gtmgbldir=${instance}/globals/mumps.gld ${instance}/gtm/mupip set -region -lock_space=400 '*' 2>&1 \
          | logger -p user.info
        retval=${PIPESTATUS[0]}
        if [ ${retval} -eq 0 ]; then
            echo "`basename ${instance}` database's lock space increased to 400 pages"
        elif [ ${retval} -eq 162 ]; then
            echo ""
            echo "Unable to increase `basename ${instance}` database's lock space to 400 pages; the database is currently in use.  Stop all processes accessing the `basename ${instance}` database, then run:"
            echo "gtm_dist=${instance}/gtm gtmgbldir=${instance}/globals/mumps.gld ${instance}/gtm/mupip set -region -lock_space=400 '*'"
            echo ""
        else
            echo ""
            echo "A problem was encountered when attempting to increase `basename ${instance}` database's lock space to 400 pages; the return code was ${retval}.  Please run the following command manually:"
            echo "gtm_dist=${instance}/gtm gtmgbldir=${instance}/globals/mumps.gld ${instance}/gtm/mupip set -region -lock_space=400 '*'"
            echo ""
        fi
    fi
done

%post -n openvista-utils
if [ $1 = 1 ]; then
    /sbin/chkconfig --add openvista-databases
    /sbin/chkconfig --add openvista
fi

/sbin/service httpd reload > /dev/null 2>&1 || :

%preun -n openvista-utils
if [ $1 = 0 ]; then
    /sbin/chkconfig --del openvista-databases
    /sbin/chkconfig --del openvista
fi

%postun -n openvista-utils
if [ $1 = 0 ]; then
    /usr/sbin/userdel openvista 2> /dev/null || :

    # if there are no users in the openvista group (other than the openvista
    # user), userdel will delete the group, but if there *are* other users, we
    # need to explicitly call groupdel
    /usr/sbin/groupdel openvista 2> /dev/null || :
fi

/sbin/service httpd reload > /dev/null 2>&1 || :





%package -n openvista-munin-plugins
Release:	1
Summary:	Munin plugins to monitor OpenVista instances
Group:		Applications/System

Requires:	/bin/bash, coreutils
Requires:	munin-node
Requires:	openvista-utils

%description -n openvista-munin-plugins
OpenVista is a cost-effective, open, trusted and complete EHR which enhances
patient safety, increases clinical and operational efficiency and provides an
opportunity to improve quality of care delivery.

Munin is a highly flexible and powerful solution used to create graphs of
virtually everything imaginable throughout your network, while still
maintaining a rattling ease of installation and configuration.

This package contains plugins for the Munin node to monitor OpenVista
instances.

%files -n openvista-munin-plugins
%defattr(-,root,root)
%doc README
%doc COPYING
%doc COPYING.AGPL
%doc NEWS
%{_sysconfdir}/munin/plugin-conf.d/openvista
%{_datadir}/munin/plugins/*

