/*
* Copyright (C) 2009 Medsphere Systems Corporation
*
* This program is free software; you can redistribute it and/or modify it
* solely under the terms of the GNU Affero General Public License version 3 as
* published by the Free Software Foundation.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License
* for more details.
* 
* You should have received a copy of the GNU Affero General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses>.
* 
* You can contact Medsphere Systems Corporation headquarters at 1917 Palomar 
* Oaks Way, Suite 200, Carlsbad, CA 92008 or at legal@medsphere.com. 
*/

#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <stdio.h>
#include "gtmxc_types.h"

#define RETRY_EINTR(x) for(;;) if ((x)!=EINTR) break
#define SEND_PID (write(fds[1], &grandchild_id, sizeof(int)))

xc_long_t gtmserver(int count, int connfd, const char *cmd) {
  int fds[2];
  int child_id;
  int grandchild_id;
  int ret_val;
  int pid;
  int result;

  ret_val = -1;
  grandchild_id = -1;

  RETRY_EINTR(result=pipe(fds));
  if (result<0) {
    perror("pipe()");
    return ret_val;
  }
  child_id=fork();
  if (child_id<0) {
    perror("child fork()");
    return ret_val;
  }
  if (child_id==0) {
    /* In child */
    RETRY_EINTR(result=close(fds[0]) );
    if (result != 0) {
      perror("child close()");
      SEND_PID;
    }
    /*
     * Fork again. Child exits and is reaped so as not to be a zombie.
     * Grandchild will be the actual client, which will be reaped by init.
     */
    grandchild_id=fork();
    if (grandchild_id<0) {
      perror("grandchild fork()");
      SEND_PID;
    }
    if (!grandchild_id) {
      /* In grandchild */
      RETRY_EINTR(dup2(connfd, STDIN_FILENO));
      RETRY_EINTR(dup2(connfd, STDOUT_FILENO));
      execlp("mumps", "mumps", "-run", cmd, (char*)0);
      /* We should never get here */
      perror("execlp() returned");
      _exit(-1); /* Do not do atexit() callbacks */
    } else {
      /* Still in child. Send grandchild pid to parent. */
      if (SEND_PID<0) {
        perror("child write()");
        _exit(-1);
      }
      if (close(fds[1]) != 0) {
        perror("child close()");
      }
      RETRY_EINTR(sleep(1000)); /* parent will kill us */
      _exit(0);
    }
  } else if (child_id) {
    /* still in parent */
    int bytes_read;
    int status;
    RETRY_EINTR(result=close(fds[1]));
    if (result != 0) {
      perror("parent close(fds[1])");
    }
    while( (bytes_read = read(fds[0], &ret_val, sizeof ret_val))==-1 && errno==EINTR) ;
    //RETRY_EINTR(bytes_read = read(fds[0], &ret_val, sizeof ret_val));
    if (bytes_read != sizeof ret_val) {
      perror("incomplete read()");
      /* ret_val = -1; */
    }
    RETRY_EINTR(result=close(fds[0]));
    if (result != 0) {
      perror("parent close(fds[0])");
    }
    kill(child_id, SIGKILL);
    waitpid(child_id, &status, 0);
    return ret_val;
  }
}
